@@ -115,7 +115,7 @@ def exit(rcode=0):
     
     print()
     
-    if sys.platform.startswith("win"):
+    if sys.platform.startswith(("win", "cygwin")):
         # different meaning of input between Py2 and Py3
         try:
             input("Press enter to continue...")
@@ -142,7 +142,7 @@ def usage(rcode=2):
     if sys.platform == "darwin":
         print("    {0} [-chxyz] [-a dir] [-b dir] [-d dir] [-f file] [-i dir]"
               " [-m name] [-p python] [--pyqt=version]".format(progName))
-    elif sys.platform.startswith("win"):
+    elif sys.platform.startswith(("win", "cygwin")):
         print("    {0} [-chxyz] [-a dir] [-b dir] [-d dir] [-f file]"
               " [--pyqt=version]"
               .format(progName))
@@ -164,7 +164,7 @@ def usage(rcode=2):
     print("               (default: {0})".format(modDir))
     print("    -f file    configuration file naming the various installation"
           " paths")
-    if not sys.platform.startswith("win"):
+    if not sys.platform.startswith(("win", "cygwin")):
         print("    -i dir     temporary install prefix")
         print("               (default: {0})".format(distDir))
     if sys.platform == "darwin":
@@ -242,7 +242,7 @@ def initGlobals():
         print("Please install the 'distutils' package first.")
         exit(5)
     
-    if sys.platform.startswith("win"):
+    if sys.platform.startswith(("win", "cygwin")):
         platBinDir = sys.exec_prefix
         if platBinDir.endswith("\\"):
             platBinDir = platBinDir[:-1]
@@ -373,20 +373,21 @@ def copyAppStreamFile(src, dst, marker):
     os.chmod(dst, 0o644)
 
 
-def wrapperName(dname, wfile):
+def wrapperNames(dname, wfile):
     """
-    Create the platform specific name for the wrapper script.
+    Create the platform specific names for the wrapper script.
     
     @param dname name of the directory to place the wrapper into
     @param wfile basename (without extension) of the wrapper script
-    @return the name of the wrapper script
+    @return the names of the wrapper scripts
     """
-    if sys.platform.startswith("win"):
-        wname = dname + "\\" + wfile + ".bat"
+    if sys.platform.startswith(("win", "cygwin")):
+        wnames = (dname + "\\" + wfile + ".cmd",
+                  dname + "\\" + wfile + ".bat")
     else:
-        wname = dname + "/" + wfile
+        wnames = (dname + "/" + wfile, )
 
-    return wname
+    return wnames
 
 
 def createPyWrapper(pydir, wfile, isGuiScript=True):
@@ -413,8 +414,8 @@ def createPyWrapper(pydir, wfile, isGuiScript=True):
         pyqt4opt = ""
     
     # all kinds of Windows systems
-    if sys.platform.startswith("win"):
-        wname = wfile + marker + ".bat"
+    if sys.platform.startswith(("win", "cygwin")):
+        wname = wfile + marker + ".cmd"
         if isGuiScript:
             wrapper = \
                 '''@echo off\n''' \
@@ -572,53 +573,10 @@ def cleanUp():
     
     # Remove the menu entry for Linux systems
     if sys.platform.startswith("linux"):
-        if os.getuid() == 0:
-            for name in ["/usr/share/pixmaps/eric.png",
-                         "/usr/share/pixmaps/ericWeb.png"]:
-                if os.path.exists(name):
-                    os.remove(name)
-            if includePythonVariant:
-                marker = PythonMarkers[sys.version_info.major]
-            else:
-                marker = ""
-            for name in [
-                "/usr/share/applications/eric6" + marker + ".desktop",
-                "/usr/share/appdata/eric6" + marker + ".appdata.xml",
-                "/usr/share/metainfo/eric6" + marker + ".appdata.xml",
-                "/usr/share/applications/eric6_webbrowser" + marker +
-                ".desktop",
-                "/usr/share/applications/eric6_browser" + marker +
-                ".desktop",
-                "/usr/share/pixmaps/eric" + marker + ".png",
-                "/usr/share/pixmaps/ericWeb" + marker + ".png",
-            ]:
-                if os.path.exists(name):
-                    os.remove(name)
-        elif os.getuid() >= 1000:
-            # it is assumed that user ids start at 1000
-            for name in ["~/.local/share/pixmaps/eric.png",
-                         "~/.local/share/pixmaps/ericWeb.png"]:
-                path = os.path.expanduser(name)
-                if os.path.exists(path):
-                    os.remove(path)
-            if includePythonVariant:
-                marker = PythonMarkers[sys.version_info.major]
-            else:
-                marker = ""
-            for name in [
-                "~/.local/share/applications/eric6" + marker + ".desktop",
-                "~/.local/share/appdata/eric6" + marker + ".appdata.xml",
-                "~/.local/share/metainfo/eric6" + marker + ".appdata.xml",
-                "~/.local/share/applications/eric6_webbrowser" + marker +
-                ".desktop",
-                "~/.local/share/applications/eric6_browser" + marker +
-                ".desktop",
-                "~/.local/share/pixmaps/eric" + marker + ".png",
-                "~/.local/share/pixmaps/ericWeb" + marker + ".png",
-            ]:
-                path = os.path.expanduser(name)
-                if os.path.exists(path):
-                    os.remove(path)
+        cleanUpLinuxSpecifics()
+    # Remove the Desktop and Start Menu entries for Windows systems
+    elif sys.platform.startswith(("win", "cygwin")):
+        cleanUpWindowsLinks()
     
     # Remove the wrapper scripts
     rem_wnames = [
@@ -645,9 +603,9 @@ def cleanUp():
             dirs.append(platBinDirOld)
         for rem_wname in rem_wnames:
             for d in dirs:
-                rwname = wrapperName(d, rem_wname)
-                if os.path.exists(rwname):
-                    os.remove(rwname)
+                for rwname in wrapperNames(d, rem_wname):
+                    if os.path.exists(rwname):
+                        os.remove(rwname)
         
         # Cleanup our config file(s)
         for name in ['eric6config.py', 'eric6config.pyc', 'eric6.pth']:
@@ -690,25 +648,127 @@ def cleanUp():
         
         if sys.platform == "darwin":
             # delete the Mac app bundle
-            try:
-                macAppBundlePath = getConfig("macAppBundlePath")
-                macAppBundleName = getConfig("macAppBundleName")
-            except AttributeError:
-                macAppBundlePath = defaultMacAppBundlePath
-                macAppBundleName = defaultMacAppBundleName
-            for bundlePath in [os.path.join(defaultMacAppBundlePath,
-                                            macAppBundleName),
-                               os.path.join(macAppBundlePath,
-                                            macAppBundleName),
-                               ]:
-                if os.path.exists(bundlePath):
-                    shutil.rmtree(bundlePath)
+            cleanUpMacAppBundle()
     except (IOError, OSError) as msg:
         sys.stderr.write(
             'Error: {0}\nTry install with admin rights.\n'.format(msg))
         exit(7)
 
 
+def cleanUpLinuxSpecifics():
+    """
+    Clean up Linux specific files.
+    """
+    if os.getuid() == 0:
+        for name in ["/usr/share/pixmaps/eric.png",
+                     "/usr/share/pixmaps/ericWeb.png"]:
+            if os.path.exists(name):
+                os.remove(name)
+        if includePythonVariant:
+            marker = PythonMarkers[sys.version_info.major]
+        else:
+            marker = ""
+        for name in [
+            "/usr/share/applications/eric6" + marker + ".desktop",
+            "/usr/share/appdata/eric6" + marker + ".appdata.xml",
+            "/usr/share/metainfo/eric6" + marker + ".appdata.xml",
+            "/usr/share/applications/eric6_webbrowser" + marker +
+            ".desktop",
+            "/usr/share/applications/eric6_browser" + marker +
+            ".desktop",
+            "/usr/share/pixmaps/eric" + marker + ".png",
+            "/usr/share/pixmaps/ericWeb" + marker + ".png",
+        ]:
+            if os.path.exists(name):
+                os.remove(name)
+    elif os.getuid() >= 1000:
+        # it is assumed that user ids start at 1000
+        for name in ["~/.local/share/pixmaps/eric.png",
+                     "~/.local/share/pixmaps/ericWeb.png"]:
+            path = os.path.expanduser(name)
+            if os.path.exists(path):
+                os.remove(path)
+        if includePythonVariant:
+            marker = PythonMarkers[sys.version_info.major]
+        else:
+            marker = ""
+        for name in [
+            "~/.local/share/applications/eric6" + marker + ".desktop",
+            "~/.local/share/appdata/eric6" + marker + ".appdata.xml",
+            "~/.local/share/metainfo/eric6" + marker + ".appdata.xml",
+            "~/.local/share/applications/eric6_webbrowser" + marker +
+            ".desktop",
+            "~/.local/share/applications/eric6_browser" + marker +
+            ".desktop",
+            "~/.local/share/pixmaps/eric" + marker + ".png",
+            "~/.local/share/pixmaps/ericWeb" + marker + ".png",
+        ]:
+            path = os.path.expanduser(name)
+            if os.path.exists(path):
+                os.remove(path)
+
+
+def cleanUpMacAppBundle():
+    """
+    Uninstall the macOS application bundle.
+    """
+    from eric6config import getConfig
+    try:
+        macAppBundlePath = getConfig("macAppBundlePath")
+        macAppBundleName = getConfig("macAppBundleName")
+    except AttributeError:
+        macAppBundlePath = defaultMacAppBundlePath
+        macAppBundleName = defaultMacAppBundleName
+    for bundlePath in [os.path.join(defaultMacAppBundlePath,
+                                    macAppBundleName),
+                       os.path.join(macAppBundlePath,
+                                    macAppBundleName),
+                       ]:
+        if os.path.exists(bundlePath):
+            shutil.rmtree(bundlePath)
+
+
+def cleanUpWindowsLinks():
+    """
+    Clean up the Desktop and Start Menu entries for Windows.
+    """
+    try:
+        from pywintypes import com_error        # __IGNORE_WARNING__
+    except ImportError:
+        # links were not created by install.py
+        return
+    
+    regPath = "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer" + \
+              "\\User Shell Folders"
+    
+    # 1. cleanup desktop links
+    regName = "Desktop"
+    desktopEntry = getWinregEntry(regName, regPath)
+    if desktopEntry:
+        desktopFolder = os.path.normpath(os.path.expandvars(desktopEntry))
+        for linkName in windowsDesktopNames():
+            linkPath = os.path.join(desktopFolder, linkName)
+            if os.path.exists(linkPath):
+                try:
+                    os.remove(linkPath)
+                except EnvironmentError:
+                    # maybe restrictions prohibited link removal
+                    print("Could not remove '{0}'.".format(linkPath))
+    
+    # 2. cleanup start menu entry
+    regName = "Programs"
+    programsEntry = getWinregEntry(regName, regPath)
+    if programsEntry:
+        programsFolder = os.path.normpath(os.path.expandvars(programsEntry))
+        eric6EntryPath = os.path.join(programsFolder, windowsProgramsEntry())
+        if os.path.exists(eric6EntryPath):
+            try:
+                shutil.rmtree(eric6EntryPath)
+            except EnvironmentError:
+                # maybe restrictions prohibited link removal
+                print("Could not remove '{0}'.".format(eric6EntryPath))
+
+
 def shutilCopy(src, dst, perm=0o644):
     """
     Wrapper function around shutil.copy() to ensure the permissions.
@@ -901,134 +961,191 @@ def installEric():
                             print("Could not install '{0}' (no permission)."
                                   .format(apiName))
     
-    # create menu entry for Linux systems
+    # Create menu entry for Linux systems
     if sys.platform.startswith("linux"):
-        if includePythonVariant:
-            marker = PythonMarkers[sys.version_info.major]
-        else:
-            marker = ""
+        createLinuxSpecifics()
     
-        if distDir:
-            dst = os.path.normpath(os.path.join(distDir, "usr/share/pixmaps"))
-            if not os.path.exists(dst):
-                os.makedirs(dst)
-            shutilCopy(
-                os.path.join(sourceDir, "icons", "default", "eric.png"),
-                os.path.join(dst, "eric" + marker + ".png"))
-            shutilCopy(
-                os.path.join(sourceDir, "icons", "default", "ericWeb48.png"),
-                os.path.join(dst, "ericWeb" + marker + ".png"))
-            dst = os.path.normpath(
-                os.path.join(distDir, "usr/share/applications"))
-            if not os.path.exists(dst):
-                os.makedirs(dst)
-            copyDesktopFile(os.path.join(sourceDir, "eric6.desktop"),
-                            os.path.join(dst, "eric6" + marker + ".desktop"),
-                            marker)
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6_webbrowser.desktop"),
-                os.path.join(dst, "eric6_webbrowser" + marker + ".desktop"),
-                marker)
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6_browser.desktop"),
-                os.path.join(dst, "eric6_browser" + marker + ".desktop"),
-                marker)
-            dst = os.path.normpath(
-                os.path.join(distDir, "usr/share/metainfo"))
-            if not os.path.exists(dst):
-                os.makedirs(dst)
-            copyAppStreamFile(
-                os.path.join(sourceDir, "eric6.appdata.xml"),
-                os.path.join(dst, "eric6" + marker + ".appdata.xml"),
-                marker)
-        elif os.getuid() == 0:
-            shutilCopy(os.path.join(
-                sourceDir, "icons", "default", "eric.png"),
-                "/usr/share/pixmaps/eric" + marker + ".png")
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6.desktop"),
-                "/usr/share/applications/eric6" + marker + ".desktop",
-                marker)
-            if os.path.exists("/usr/share/metainfo"):
-                copyAppStreamFile(
-                    os.path.join(sourceDir, "eric6.appdata.xml"),
-                    "/usr/share/metainfo/eric6" + marker + ".appdata.xml",
-                    marker)
-            elif os.path.exists("/usr/share/appdata"):
-                copyAppStreamFile(
-                    os.path.join(sourceDir, "eric6.appdata.xml"),
-                    "/usr/share/appdata/eric6" + marker + ".appdata.xml",
-                    marker)
-            shutilCopy(os.path.join(
-                sourceDir, "icons", "default", "ericWeb48.png"),
-                "/usr/share/pixmaps/ericWeb" + marker + ".png")
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6_webbrowser.desktop"),
-                "/usr/share/applications/eric6_webbrowser" + marker +
-                ".desktop",
-                marker)
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6_browser.desktop"),
-                "/usr/share/applications/eric6_browser" + marker +
-                ".desktop",
-                marker)
-        elif os.getuid() >= 1000:
-            # it is assumed, that user ids start at 1000
-            localPath = os.path.join(os.path.expanduser("~"),
-                                     ".local", "share")
-            # create directories first
-            for directory in [os.path.join(localPath, name)
-                              for name in ("pixmaps", "applications",
-                                           "metainfo", "appdata")]:
-                if not os.path.isdir(directory):
-                    os.makedirs(directory)
-            # now copy the files
-            shutilCopy(
-                os.path.join(sourceDir, "icons", "default", "eric.png"),
-                os.path.join(localPath, "pixmaps", "eric" + marker + ".png"))
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6.desktop"),
-                os.path.join(localPath, "applications",
-                             "eric6" + marker + ".desktop"),
-                marker)
-            copyAppStreamFile(
-                os.path.join(sourceDir, "eric6.appdata.xml"),
-                os.path.join(localPath, "metainfo",
-                             "eric6" + marker + ".appdata.xml"),
-                marker)
-            copyAppStreamFile(
-                os.path.join(sourceDir, "eric6.appdata.xml"),
-                os.path.join(localPath, "appdata",
-                             "eric6" + marker + ".appdata.xml"),
-                marker)
-            shutilCopy(
-                os.path.join(sourceDir, "icons", "default", "ericWeb48.png"),
-                os.path.join(localPath, "pixmaps",
-                             "ericWeb" + marker + ".png"))
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6_webbrowser.desktop"),
-                os.path.join(localPath, "applications",
-                             "eric6_webbrowser" + marker + ".desktop"),
-                marker)
-            copyDesktopFile(
-                os.path.join(sourceDir, "eric6_browser.desktop"),
-                os.path.join(localPath, "applications",
-                             "eric6_browser" + marker + ".desktop"),
-                marker)
+    # Create Desktop and Start Menu entries for Windows systems
+    elif sys.platform.startswith(("win", "cygwin")):
+        createWindowsLinks()
     
     # Create a Mac application bundle
-    if sys.platform == "darwin":
+    elif sys.platform == "darwin":
         createMacAppBundle(cfg['ericDir'])
     
     return 0
 
 
+def createLinuxSpecifics():
+    """
+    Install Linux specific files.
+    """
+    global distDir, sourceDir, includePythonVariant
+    
+    if includePythonVariant:
+        marker = PythonMarkers[sys.version_info.major]
+    else:
+        marker = ""
+
+    if distDir:
+        dst = os.path.normpath(os.path.join(distDir, "usr/share/pixmaps"))
+        if not os.path.exists(dst):
+            os.makedirs(dst)
+        shutilCopy(
+            os.path.join(sourceDir, "icons", "default", "eric.png"),
+            os.path.join(dst, "eric" + marker + ".png"))
+        shutilCopy(
+            os.path.join(sourceDir, "icons", "default", "ericWeb48.png"),
+            os.path.join(dst, "ericWeb" + marker + ".png"))
+        dst = os.path.normpath(
+            os.path.join(distDir, "usr/share/applications"))
+        if not os.path.exists(dst):
+            os.makedirs(dst)
+        copyDesktopFile(os.path.join(sourceDir, "eric6.desktop"),
+                        os.path.join(dst, "eric6" + marker + ".desktop"),
+                        marker)
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6_webbrowser.desktop"),
+            os.path.join(dst, "eric6_webbrowser" + marker + ".desktop"),
+            marker)
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6_browser.desktop"),
+            os.path.join(dst, "eric6_browser" + marker + ".desktop"),
+            marker)
+        dst = os.path.normpath(
+            os.path.join(distDir, "usr/share/metainfo"))
+        if not os.path.exists(dst):
+            os.makedirs(dst)
+        copyAppStreamFile(
+            os.path.join(sourceDir, "eric6.appdata.xml"),
+            os.path.join(dst, "eric6" + marker + ".appdata.xml"),
+            marker)
+    elif os.getuid() == 0:
+        shutilCopy(os.path.join(
+            sourceDir, "icons", "default", "eric.png"),
+            "/usr/share/pixmaps/eric" + marker + ".png")
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6.desktop"),
+            "/usr/share/applications/eric6" + marker + ".desktop",
+            marker)
+        if os.path.exists("/usr/share/metainfo"):
+            copyAppStreamFile(
+                os.path.join(sourceDir, "eric6.appdata.xml"),
+                "/usr/share/metainfo/eric6" + marker + ".appdata.xml",
+                marker)
+        elif os.path.exists("/usr/share/appdata"):
+            copyAppStreamFile(
+                os.path.join(sourceDir, "eric6.appdata.xml"),
+                "/usr/share/appdata/eric6" + marker + ".appdata.xml",
+                marker)
+        shutilCopy(os.path.join(
+            sourceDir, "icons", "default", "ericWeb48.png"),
+            "/usr/share/pixmaps/ericWeb" + marker + ".png")
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6_webbrowser.desktop"),
+            "/usr/share/applications/eric6_webbrowser" + marker +
+            ".desktop",
+            marker)
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6_browser.desktop"),
+            "/usr/share/applications/eric6_browser" + marker +
+            ".desktop",
+            marker)
+    elif os.getuid() >= 1000:
+        # it is assumed, that user ids start at 1000
+        localPath = os.path.join(os.path.expanduser("~"),
+                                 ".local", "share")
+        # create directories first
+        for directory in [os.path.join(localPath, name)
+                          for name in ("pixmaps", "applications",
+                                       "metainfo", "appdata")]:
+            if not os.path.isdir(directory):
+                os.makedirs(directory)
+        # now copy the files
+        shutilCopy(
+            os.path.join(sourceDir, "icons", "default", "eric.png"),
+            os.path.join(localPath, "pixmaps", "eric" + marker + ".png"))
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6.desktop"),
+            os.path.join(localPath, "applications",
+                         "eric6" + marker + ".desktop"),
+            marker)
+        copyAppStreamFile(
+            os.path.join(sourceDir, "eric6.appdata.xml"),
+            os.path.join(localPath, "metainfo",
+                         "eric6" + marker + ".appdata.xml"),
+            marker)
+        copyAppStreamFile(
+            os.path.join(sourceDir, "eric6.appdata.xml"),
+            os.path.join(localPath, "appdata",
+                         "eric6" + marker + ".appdata.xml"),
+            marker)
+        shutilCopy(
+            os.path.join(sourceDir, "icons", "default", "ericWeb48.png"),
+            os.path.join(localPath, "pixmaps",
+                         "ericWeb" + marker + ".png"))
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6_webbrowser.desktop"),
+            os.path.join(localPath, "applications",
+                         "eric6_webbrowser" + marker + ".desktop"),
+            marker)
+        copyDesktopFile(
+            os.path.join(sourceDir, "eric6_browser.desktop"),
+            os.path.join(localPath, "applications",
+                         "eric6_browser" + marker + ".desktop"),
+            marker)
+
+
+def createWindowsLinks():
+    """
+    Create Desktop and Start Menu links.
+    """
+    try:
+        # check, if pywin32 is available
+        from win32com.client import Dispatch    # __IGNORE_WARNING__
+    except ImportError:
+        print(
+            "\nThe Python package 'pywin32' is not installed. Desktop and"
+            " Start Menu entries will not be created."
+        )
+        return
+    
+    regPath = "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer" + \
+              "\\User Shell Folders"
+    
+    # 1. create desktop shortcuts
+    regName = "Desktop"
+    desktopFolder = os.path.normpath(
+            os.path.expandvars(getWinregEntry(regName, regPath)))
+    for linkName, targetPath, iconPath in windowsDesktopEntries():
+        linkPath = os.path.join(desktopFolder, linkName)
+        createWindowsShortcut(linkPath, targetPath, iconPath)
+    
+    # 2. create start menu entry and shortcuts
+    regName = "Programs"
+    programsEntry = getWinregEntry(regName, regPath)
+    if programsEntry:
+        programsFolder = os.path.normpath(os.path.expandvars(programsEntry))
+        eric6EntryPath = os.path.join(programsFolder, windowsProgramsEntry())
+        if not os.path.exists(eric6EntryPath):
+            try:
+                os.makedirs(eric6EntryPath)
+            except EnvironmentError:
+                # maybe restrictions prohibited link creation
+                return
+        
+        for linkName, targetPath, iconPath in windowsDesktopEntries():
+            linkPath = os.path.join(eric6EntryPath, linkName)
+            createWindowsShortcut(linkPath, targetPath, iconPath)
+
+
 def createMacAppBundle(pydir):
     """
     Create a Mac application bundle.
 
     @param pydir the name of the directory where the Python script will
-        eventually be installed (string)
+        eventually be installed
+    @type str
     """
     global cfg, sourceDir, macAppBundleName, macPythonExe, macAppBundlePath, \
         pyqtVariant
@@ -1371,7 +1488,10 @@ def doDependancyChecks():
             "PyQt5.QtSql", "PyQt5.QtSvg", "PyQt5.QtWidgets",
         ]
         altModulesList = [
-            ("PyQt5.QtWebEngineWidgets", "PyQt5.QtWebKitWidgets")
+            # Tuple with alternatives, flag indicating it's ok to not be
+            # available (e.g. for 32-Bit Windows)
+            (("PyQt5.QtWebEngineWidgets", "PyQt5.QtWebKitWidgets"),
+             sys.maxsize <= 2**32),
         ]
     # check mandatory modules
     modulesOK = True
@@ -1389,7 +1509,7 @@ def doDependancyChecks():
     # check mandatory modules with alternatives
     if altModulesList:
         altModulesOK = True
-        for altModules in altModulesList:
+        for altModules, forcedOk in altModulesList:
             modulesOK = False
             for altModule in altModules:
                 name = altModule.split(".")[1]
@@ -1400,7 +1520,7 @@ def doDependancyChecks():
                     break
                 except ImportError:
                     pass
-            if not modulesOK:
+            if not modulesOK and not forcedOk:
                 altModulesOK = False
                 print('Sorry, please install {0}.'
                       .format(" or ".join(altModules)))
@@ -1408,7 +1528,7 @@ def doDependancyChecks():
             exit(1)
     
     # determine the platform dependent black list
-    if sys.platform.startswith("win"):
+    if sys.platform.startswith(("win", "cygwin")):
         PlatformBlackLists = PlatformsBlackLists["windows"]
     elif sys.platform.startswith("linux"):
         PlatformBlackLists = PlatformsBlackLists["linux"]
@@ -1581,6 +1701,122 @@ def prepareInfoFile(fileName):
         shutil.copy(fileName + ".orig", fileName)
 
 
+def getWinregEntry(name, path):
+    """
+    Function to get an entry from the Windows Registry.
+    
+    @param name variable name
+    @type str
+    @param path registry path of the variable
+    @type str
+    @return value of requested registry variable
+    @rtype any
+    """
+    try:
+        import _winreg as winreg
+    except ImportError:
+        try:
+            import winreg
+        except ImportError:
+            return None
+    
+    try:
+        registryKey = winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0,
+                                     winreg.KEY_READ)
+        value, _ = winreg.QueryValueEx(registryKey, name)
+        winreg.CloseKey(registryKey)
+        return value
+    except WindowsError:
+        return None
+
+
+def createWindowsShortcut(linkPath, targetPath, iconPath):
+    """
+    Create Windows shortcut.
+    
+    @param linkPath path of the shortcut file
+    @type str
+    @param targetPath path the shortcut shall point to
+    @type str
+    @param iconPath path of the icon file
+    @type str
+    """
+    from win32com.client import Dispatch
+    from pywintypes import com_error
+    
+    try:
+        shell = Dispatch('WScript.Shell')
+        shortcut = shell.CreateShortCut(linkPath)
+        shortcut.Targetpath = targetPath
+        shortcut.WorkingDirectory = os.path.dirname(targetPath)
+        shortcut.IconLocation = iconPath
+        shortcut.save()
+    except com_error:
+        # maybe restrictions prohibited link creation
+        pass
+
+
+def windowsDesktopNames():
+    """
+    Function to generate the link names for the Windows Desktop.
+    
+    @return list of desktop link names
+    @rtype list of str
+    """
+    return [e[0] for e in windowsDesktopEntries()]
+
+
+def windowsDesktopEntries():
+    """
+    Function to generate data for the Windows Desktop links.
+    
+    @return list of tuples containing the desktop link name,
+        the link target and the icon target
+    @rtype list of tuples of (str, str, str)
+    """
+    global cfg, includePythonVariant
+    
+    if includePythonVariant:
+        marker = PythonMarkers[sys.version_info.major]
+    else:
+        marker = ""
+    
+    majorVersion, minorVersion = sys.version_info[:2]
+    entriesTemplates = [
+        ("eric6 (Python {0}.{1}).lnk",
+         os.path.join(cfg["bindir"], "eric6" + marker + ".cmd"),
+         os.path.join(cfg["ericPixDir"], "eric6.ico")),
+    ]
+    if sys.version_info.major == 2:
+        entriesTemplates.append((
+            "eric6 Browser (Python {0}.{1}).lnk",
+            os.path.join(cfg["bindir"], "eric6_webbrowser" + marker + ".cmd"),
+            os.path.join(cfg["ericPixDir"], "ericWeb48.ico")
+        ))
+    else:
+        entriesTemplates.append((
+            "eric6 Browser (Python {0}.{1}).lnk",
+            os.path.join(cfg["bindir"], "eric6_browser" + marker + ".cmd"),
+            os.path.join(cfg["ericPixDir"], "ericWeb48.ico")
+        ))
+    
+    return [
+        (e[0].format(majorVersion, minorVersion), e[1], e[2])
+        for e in entriesTemplates
+    ]
+
+
+def windowsProgramsEntry():
+    """
+    Function to generate the name of the Start Menu top entry.
+    
+    @return name of the Start Menu top entry
+    @rtype str
+    """
+    majorVersion, minorVersion = sys.version_info[:2]
+    return "eric6 (Python {0}.{1})".format(majorVersion, minorVersion)
+
+
 def main(argv):
     """
     The main function of the script.
@@ -1609,7 +1845,7 @@ def main(argv):
     initGlobals()
 
     try:
-        if sys.platform.startswith("win"):
+        if sys.platform.startswith(("win", "cygwin")):
             optlist, args = getopt.getopt(
                 argv[1:], "chxyza:b:d:f:", ["help", "pyqt=", "noapis"])
         elif sys.platform == "darwin":
